import greenfoot.*;
import java.util.List;
import java.util.ArrayList;

/**
 * A collection of LayerActors.<p>
 * 
 * A layer has a certain width and height.
 * You enter that when you create one, after
 * then you cannot change them:
 * <pre>
 * Layer l = new Layer(width, height);
 * </pre>
 * 
 * A layer needs to be added to a LayerWorld,
 * in order to do anything.<p>
 * 
 * The location of the layer is changeable,
 * and is first entered when added to a
 * layer world:
 * <pre>
 * LayerWorld world = ...;
 * world.addLayer(layer, x, y);
 * </pre>
 * 
 * @author Sven van Nigtevecht
 * @version 1.0
 */
public class Layer
{
    private final ArrayList<LayerActor> actors;
    private final int width,height;
    private final boolean bounded;
    
    private LayerWorld world;
    private int x,y;
    private boolean visible;
    
    /**
     * Create a new Layer.
     * @param width The width of the layer.
     * @param height The height of the layer.
     */
    public Layer(int width, int height)
    {
        this(width, height, true);
    }
    
    /**
     * Create a new Layer.
     * @param width The width of the layer.
     * @param height The height of the layer.
     * @param bounded Are the actors limited to the
     * layer's boundary?
     */
    public Layer(int width, int height, boolean bounded)
    {
        this.actors = new ArrayList<LayerActor>();
        this.width = width;
        this.height = height;
        this.bounded = bounded;
        
        this.world = null;
        this.x = 0;
        this.y = 0;
        this.visible = true;
    }
    
    /** PUBLIC API: */
    
    /** Locations: */
    
    /**
     * Adds a LayerActor to the layer.
     * @param actor The actor which will be added
     * to the layer.
     * @param x The x coördinate the actor will be added at.
     * @param y The y coördinate the actor will be added at.
     */
    public void add(LayerActor actor, int x, int y)
    {
        testWorld();
        int newX = boundedXForX(x);
        int newY = boundedYForY(y);
        actor.setLayer(this);
        world.addObject(actor,newX,newY);
        actors.add(actor);
    }
    
    /**
     * Returns the x coördinate of the layer.
     */
    public int getX()
    {
        return x;
    }
    
    /**
     * Returns the y coördinate of the layer.
     */
    public int getY()
    {
        return y;
    }
    
    /**
     * Sets the location of the layer.
     * @param newX The x coördinate of the layer's
     * new location.
     * @param newY The y coördinate of the layer's
     * new location.
     */
    public void setLocation(int newX, int newY)
    {
        testWorld();
        
        if (world.isBounded()) {
            if (newX < 0)
                newX = 0;
            else if (newX > world.getWidth() -width)
                newX = world.getWidth() -width;
            if (newY < 0)
                newY = 0;
            else if (newY > world.getHeight() -height)
                newY = world.getHeight() -height;
        }
        
        int dx = newX -this.x;
        int dy = newY -this.y;
        this.x = newX;
        this.y = newY;
        for (Actor a : actors) {
            a.setLocation(a.getX() +dx, a.getY() +dy);
        }
    }
    
    /**
     * Returns the width of the layer.
     */
    public int getWidth()
    {
        return width;
    }
    
    /**
     * Returns the height of the layer.
     */
    public int getHeight()
    {
        return height;
    }
    
    /**
     * Are the actors limited to the layer's boundary?
     */
    public boolean isBounded()
    {
        return bounded;
    }
    
    
    /** Visibility: */
    
    /**
     * Sets the layer is visible or not.
     * @param visible The new state of the layer.
     */
    public void setVisible(boolean visible)
    {
        if (this.visible == visible) return;
        this.visible = visible;
        if (visible) {
            for (LayerActor a : actors) {
                world.addObject(a,a.oldX,a.oldY);
            }
        } else {
            for (LayerActor a : actors) {
                world.removeObject(a);
            }
        }
    }
    
    /**
     * Is the layer visible or not?
     */
    public boolean isVisible()
    {
        return visible;
    }
    
    /** Other things: */
    
    /**
     * Returns the world the layer is in.
     */
    public LayerWorld getWorld()
    {
        testWorld();
        return world;
    }
    
    /**
     * Returns a newly instancieted List
     * which contains all the actors the layer
     * has.
     */
    public List<LayerActor> getActors()
    {
        return new ArrayList<LayerActor>(actors);
    }
    
    /** SYSTEM & PRIVATE API: */
    
    /**
     * Used by the sytem: don't touch!
     */
    void setWorld(LayerWorld world)
    {
        this.world = world;
    }
    
    private void testWorld()
    {
        if (this.world == null)
            throw new IllegalStateException("The layer"+
            " isn't inserted in a world (yet)");
    }
    
    private int boundedXForX(int input)
    {
        if (!bounded) return input;
        if (input < 0)
            return 0;
        else if (input > width)
            return width -1;
        return input;
    }
    
    private int boundedYForY(int input)
    {
        if (!bounded) return input;
        if (input < 0)
            return 0;
        else if (input > height)
            return height -1;
        return input;
    }
}